<div class="d-flex align-items-center justify-content-between {{css_class}}">
  <div class="{{header_class}} d-flex align-items-center">
    {{header_text}} 
    {% if badge_text %}<span class='text-white rounded-pill bg-{{badge_class}} ms-2 px-3'>{{badge_text}}</span>{% endif %}
  </div>
  {% block controls %}
  {% endblock %}  
</div>